// MonthlyReport model definition
import { Sequelize, DataTypes, Model, Optional, Op, fn, col } from 'sequelize';
import moment from 'moment'; 
import { Transaction } from './Transaction'; // Assuming you have a Transaction model
import { Account } from './Account';
import { getFirstLastDayOfMonth } from '../helper';

interface MonthlyReportAttributes {
    MonthlyReportID: number;
    Month: number;
    Year: number;
    TotalAmount: number;

    // Other report fields...
    AccountID: number;
}

interface MonthlyReportCreationAttributes extends Optional<MonthlyReportAttributes, 'MonthlyReportID'> { }

class MonthlyReport extends Model<MonthlyReportAttributes, MonthlyReportCreationAttributes> implements MonthlyReportAttributes {
    public MonthlyReportID!: number;
    public Month!: number;
    public Year!: number;
    public TotalAmount!: number;
    public AccountID!: number;

    static initialize(sequelize: Sequelize): void {
        MonthlyReport.init(
            {
                MonthlyReportID: {
                    type: DataTypes.INTEGER,
                    primaryKey: true,
                    autoIncrement: true,
                },
                Month: {
                    type: DataTypes.INTEGER,
                    allowNull: false,
                },
                Year: {
                    type: DataTypes.INTEGER,
                    allowNull: false,
                },
                TotalAmount: {
                    type: DataTypes.FLOAT,
                    allowNull: false,
                },
                AccountID: {
                    type: DataTypes.INTEGER,
                    allowNull: false,
                }
                // Other report fields...
            },
            {
                sequelize,
                modelName: 'MonthlyReport',
            }
        );
    }

    static associate(models: { Transaction: typeof Transaction, Account: typeof Account }): void {
        MonthlyReport.hasMany(models.Transaction, { foreignKey: 'MonthlyReportID' }); // A report can have multiple transactions
        MonthlyReport.belongsTo(models.Account)
    }

    static async generateMonthlyReport(month: number, year: number, AccountID: number|null = null): Promise<void> {
        try {
            moment.locale("de")


            let m = moment().set({year:year,month:month-1,date:15});
            let { startDate, endDate } = getFirstLastDayOfMonth(m);

            // Fetch transaction data and calculate total amount
            let transactions = null;
            if(AccountID==null){
                transactions = await Transaction.findAll({
                    where: {
                        Date: {
                            [Op.between]: [startDate, endDate],
                        }
                    },
                });
            }else{
                transactions = await Transaction.findAll({
                    where: {
                        Date: {
                            [Op.between]: [startDate, endDate],
                        },
                        AccountID: AccountID
                    },
                });
            }
            console.log(transactions)


            for(let i=0;i<transactions.length;i++){
                await Transaction.update({
                    Locked:true
                },{
                    where:{
                        TransactionID:transactions[i].TransactionID
                    }
                })
            }

            const totalAmount = transactions.reduce((total, transaction) => total + transaction.Amount, 0);


            console.log(totalAmount)
            // Create a MonthlyReport entry with the generated data
            await MonthlyReport.create({
                Month: month,
                Year: year,
                TotalAmount: totalAmount,
                AccountID: 0
            });

            console.log(`Monthly report generated for ${month}/${year}`);
        } catch (err) {
            console.error('Error generating monthly report:', err);
        } finally {

        }
    }
}

export { MonthlyReport };