import { Sequelize, DataTypes, Model, Optional, Attributes, CreateOptions, ModelStatic } from 'sequelize';
import { User } from './User';
import { Category } from './Category';
import { Account } from './Account';
import { HookReturn } from 'sequelize/types/hooks';
import { MonthlyReport } from './MonthlyReport';



// Transaction attributes interface
interface TransactionAttributes {
    TransactionID: number;
    Date: Date;
    Description: string | null;
    UserID: number;
    Amount: number;
    Type: 'income' | 'expense' | 'transfer';
    AccountID: number;
    Refound:boolean,
    Locked: boolean
    // Other fields...

}

// Transaction creation attributes (optional fields)
interface TransactionCreationAttributes extends Optional<TransactionAttributes, 'TransactionID'> { }

// Define the Transaction model
class Transaction extends Model<TransactionAttributes, TransactionCreationAttributes> implements TransactionAttributes {
    public TransactionID!: number;
    public Date!: Date;
    public Description!: string | null;
    public Amount!: number;
    public Type!: 'income' | 'expense' | 'transfer';
    public Refound!: boolean;
    public Locked!:boolean;
    // Other fields...

    public UserID!: number;
    public AccountID!: number;
    public CategoryID!: number;

    static initialize(sequelize: Sequelize): void {
        Transaction.init(
            {
                TransactionID: {
                    type: DataTypes.INTEGER,
                    primaryKey: true,
                    autoIncrement: true,
                },
                Date: {
                    type: DataTypes.DATE,
                    allowNull: false,
                    defaultValue: new Date()
                },
                Description: {
                    type: DataTypes.STRING,
                    allowNull: true,
                },
                UserID: {
                    type: DataTypes.INTEGER,
                    allowNull: false,
                },
                AccountID: {
                    type: DataTypes.INTEGER,
                    allowNull: false,
                    defaultValue: 1,
                },
                Amount: {
                    type: DataTypes.FLOAT,
                    allowNull: false,
                },
                Type: {
                    type: DataTypes.ENUM('income', 'expense','transfer'),
                    allowNull: false,
                },
                Refound:{
                    type:DataTypes.BOOLEAN,
                    defaultValue:false
                },
                Locked:{
                    type:DataTypes.BOOLEAN,
                    defaultValue:false
                }
                // Other fields...
            },
            {
                sequelize,
                modelName: 'Transaction',
                // Other model options...
                hooks: {
                    afterCreate: (attributes: Transaction, options: CreateOptions<TransactionAttributes>) => {
                        console.log(attributes)
                    },
                    beforeCreate(attributes: Transaction, options: CreateOptions<TransactionAttributes>) {
                        if (attributes.Type == "expense") {
                            attributes.Amount = -0 - attributes.Amount;
                        }
                    }
                }
            }
        );
    }
    static associate(models: { User: typeof User; Category: typeof Category; Account: typeof Account; MonthlyReport: typeof MonthlyReport }): void {
        Transaction.belongsTo(models.User); // A transaction belongs to a user
        Transaction.belongsTo(models.Category, { foreignKey: 'CategoryID' }); // A transaction belongs to a category
        Transaction.belongsTo(models.Account, { foreignKey: 'AccountID' }); // A transaction belongs to an account
    }

    static async getTotalAmountForUser(userId: number): Promise<number> {
        const result = await Transaction.sum('Amount', { where: { UserID: userId } });
        return result || 0;
    }

    static async transferAmountFromTo(userID: number, fromAccountID: number = 1, toAccountID: number = 1, Amount: number = 0,CategoryID:number = 1){
        console.log(`User: ${userID}, fromAccount: ${fromAccountID}, toAccount${toAccountID}, Amount: ${Amount}`)
        await Transaction.create({
            Amount:-Amount,
            Date:new Date(),
            Type:"transfer",
            //@ts-ignore
            UserID: userID, // Associate the transaction with the user
            //@ts-ignore
            CategoryID: CategoryID,
            AccountID: fromAccountID
        })
        await Transaction.create({
            Amount:Amount,
            Date:new Date(),
            Type:"transfer",
            //@ts-ignore
            UserID: userID, // Associate the transaction with the user
            //@ts-ignore
            CategoryID: CategoryID,
            AccountID: toAccountID
        })
    }

    static async changeTransaction(id:number,amount:number,Description:string, CategoryID:number, AccountID:number){
        let foundTransaction = await Transaction.findOne({where:{
            TransactionID:id
        }})
        if(foundTransaction!=null){
            if(foundTransaction.Locked == false){

            }else{
                
            }
        }
    }
}


export { Transaction };